package com.ruoyi.system.service.impl;

import com.ruoyi.system.kernel.ServerLogKernel;
import com.ruoyi.system.domain.dto.DockerLogsParams;
import com.ruoyi.common.core.utils.DateUtils;
import org.springframework.scheduling.annotation.Async;
import com.alibaba.fastjson2.JSONWriter;
import com.ruoyi.system.utils.RequestUtil;
import com.alibaba.fastjson2.JSONObject;
import java.util.Iterator;
import java.util.stream.Collector;
import java.util.stream.Collectors;
import com.ruoyi.system.utils.jsch.ServiceShell;
import com.ruoyi.system.utils.dbsync.runner.Syncer;
import com.ruoyi.system.domain.dto.UpdateBaseParams;
import org.apache.commons.lang3.tuple.Triple;
import com.ruoyi.system.utils.SSLExpirationChecker;
import com.ruoyi.system.domain.dto.DeploySSLParams;
import java.util.Objects;
import org.springframework.transaction.annotation.Transactional;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.common.core.domain.R;
import java.util.List;
import com.ruoyi.system.api.model.LoginUser;
import com.ruoyi.system.domain.SnServerLog;
import com.ruoyi.system.enums.TaskTypeEnum;
import com.ruoyi.system.api.domain.SysFile;
import com.ruoyi.common.core.constant.Constants;
import com.ruoyi.common.core.domain.StrFile;
import com.ruoyi.common.core.utils.file.FileUtils;
import com.ruoyi.system.domain.SnTemplate;
import com.ruoyi.common.core.utils.ServletUtils;
import org.apache.commons.lang3.tuple.Pair;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.system.domain.SnServerTask;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.system.domain.SnServer;
import java.util.concurrent.Executors;
import java.util.concurrent.ExecutorService;
import org.springframework.beans.factory.annotation.Value;
import com.ruoyi.system.service.ILaFilterUaService;
import com.ruoyi.system.kernel.SslCountLimit;
import com.ruoyi.system.kernel.SslNginxConfigDealKernel;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.system.service.ISysDictDataService;
import com.ruoyi.system.kernel.CreateShellTaskKernel;
import com.ruoyi.system.kernel.DomainValidKernel;
import com.ruoyi.system.mq.ServerTaskSender;
import com.ruoyi.system.api.RemoteFileService;
import com.ruoyi.system.service.ISysConfigService;
import com.ruoyi.system.mapper.SnServerTemplateMapper;
import com.ruoyi.system.mapper.SnServerLogMapper;
import com.ruoyi.system.mapper.SnServerTaskMapper;
import com.ruoyi.system.mapper.SnTemplateMapper;
import com.ruoyi.common.security.service.TokenService;
import org.springframework.beans.factory.annotation.Autowired;
import com.ruoyi.system.mapper.SnServerMapper;
import org.springframework.stereotype.Service;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import com.ruoyi.system.service.ISnServerService;

@RefreshScope
@Service
public class SnServerServiceImpl implements ISnServerService
{
    @Autowired
    private SnServerMapper snServerMapper;
    @Autowired
    private TokenService tokenService;
    @Autowired
    private SnTemplateMapper snTemplateMapper;
    @Autowired
    private SnServerTaskMapper snServerTaskMapper;
    @Autowired
    private SnServerLogMapper snServerLogMapper;
    @Autowired
    private SnServerTemplateMapper snServerTemplateMapper;
    @Autowired
    private ISysConfigService configService;
    @Autowired
    private RemoteFileService remoteFileService;
    @Autowired
    private ServerTaskSender serverTaskSender;
    @Autowired
    private DomainValidKernel domainValidKernel;
    @Autowired
    private CreateShellTaskKernel createShellTaskKernel;
    @Autowired
    private ISysDictDataService dictDataService;
    @Autowired
    private ISysUserService sysUserService;
    @Autowired
    private SslNginxConfigDealKernel sslNginxConfigDealKernel;
    @Autowired
    private SslCountLimit sslCountLimit;
    @Autowired
    private ILaFilterUaService filterUaService;
    @Value("${baseNginxUrl}")
    public String baseNginxUrl;
    @Value("${baseYamlUrl}")
    public String baseYamlUrl;
    @Value("${baseSqlUrl}")
    public String baseSqlUrl;
    @Value("${baseSshUrl}")
    public String baseSshUrl;
    @Value("${checkLicenseUrl}")
    public String checkLicenseUrl;
    @Value("${dbSync.open}")
    public Boolean dbSyncOpen;
    ExecutorService executor;
    
    public SnServerServiceImpl() {
        this.executor = Executors.newSingleThreadExecutor();
    }
    
    public AjaxResult deployBasicEnv(final SnServer snParam) {
        final Pair<SnServerTask, String> pair = this.deployBasic(snParam);
        if (pair.getLeft() != null) {
            this.serverTaskSender.sendOrderMessage(pair.getLeft().getTaskId());
        }
        if (StringUtils.isNotEmpty((String)pair.getRight())) {
            return AjaxResult.error((String)pair.getRight());
        }
        return AjaxResult.success("部署中，请稍后查看服务器任务状态");
    }
    
    @Transactional
    public Pair<SnServerTask, String> deployBasic(final SnServer snParam) {
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        final SnServer server = this.selectSnServerByServerId(snParam.getServerId());
        if (server == null) {
            return Pair.of((SnServerTask)null, "未查询到服务器信息");
        }
        final String domain = snParam.getDomain();
        final AjaxResult result = this.domainValidKernel.checkDomain(domain, server.getIp(), server.getServerId());
        if (!result.isSuccess()) {
            return Pair.of((SnServerTask)null, result.get((Object)"msg"));
        }
        final SnTemplate params = new SnTemplate();
        params.getParams().put("limitNum", 1);
        params.setCategory(Integer.valueOf(0));
        final List<SnTemplate> backs = this.snTemplateMapper.selectSnTemplateList(params);
        if (backs.isEmpty()) {
            return Pair.of((SnServerTask)null, "未获取到基础环境后台包，请联系管理员");
        }
        final SnTemplate back = backs.get(0);
        params.setCategory(Integer.valueOf(1));
        final List<SnTemplate> fonts = this.snTemplateMapper.selectSnTemplateList(params);
        if (fonts.isEmpty()) {
            return Pair.of((SnServerTask)null, "未获取到基础环境UI包，请联系管理员");
        }
        final SnTemplate font = fonts.get(0);
        final String dbPsd = "R8mrVY7Em85mO5O";
        String yamlConfig = FileUtils.readFile(this.baseYamlUrl);
        yamlConfig = yamlConfig.replace("#{sync-client-image}", back.getPackUrl());
        StrFile strFile = new StrFile();
        strFile.setContent(yamlConfig);
        strFile.setFileName("docker-compose.yml");
        strFile.setContentType("application/x-yaml");
        final R<SysFile> r = (R<SysFile>)this.remoteFileService.uploadStrToMinio(strFile);
        if (r.getCode() != Constants.SUCCESS) {
            return Pair.of((SnServerTask)null, r.getMsg());
        }
        final String yamlUrl = ((SysFile)r.getData()).getUrl();
        String nginxConfig = FileUtils.readFile(this.baseNginxUrl);
        nginxConfig = nginxConfig.replace("#{server_name}", domain);
        nginxConfig = nginxConfig.replace("#{base_root}", "");
        nginxConfig = nginxConfig.replace("#{open_80}", "");
        strFile = new StrFile();
        strFile.setContent(nginxConfig);
        strFile.setFileName(domain + ".conf");
        strFile.setContentType("text/plain");
        final R<SysFile> r2 = (R<SysFile>)this.remoteFileService.uploadStrToMinio(strFile);
        if (r2.getCode() != Constants.SUCCESS) {
            return Pair.of((SnServerTask)null, r2.getMsg());
        }
        final String nginxUrl = ((SysFile)r2.getData()).getUrl();
        String sshConfig = FileUtils.readFile(this.baseSshUrl);
        sshConfig = sshConfig.replace("#{docker-compose}", yamlUrl);
        sshConfig = sshConfig.replace("#{font-back-pack}", font.getPackUrl());
        sshConfig = sshConfig.replace("#{font-back-name}", domain);
        sshConfig = sshConfig.replace("#{font-back-nginx}", nginxUrl);
        strFile = new StrFile();
        strFile.setContent(sshConfig);
        strFile.setFileName("install.sh");
        strFile.setContentType("text/plain");
        final R<SysFile> r3 = (R<SysFile>)this.remoteFileService.uploadStrToMinio(strFile);
        if (r3.getCode() != Constants.SUCCESS) {
            return Pair.of((SnServerTask)null, r3.getMsg());
        }
        final String sshUrl = ((SysFile)r3.getData()).getUrl();
        final SnServer upObj = new SnServer();
        upObj.setServerId(server.getServerId());
        upObj.setDomain(domain);
        upObj.setVersion(this.configService.selectConfigByKey("sync-client-version"));
        upObj.setBackVersion(back.getVersion());
        upObj.setUiVersion(font.getVersion());
        upObj.setUaVersion("0.0");
        upObj.setNginxCnf(nginxUrl);
        upObj.setStatus(Integer.valueOf(1));
        upObj.setEnDbPsd(dbPsd);
        final SnServerTask task = new SnServerTask(server, loginUser);
        task.setTaskName("安装服务器基础环境");
        task.setType(TaskTypeEnum.installBase.getType());
        task.setProcess(snParam.getProcess());
        this.snServerTaskMapper.insertSnServerTask(task);
        if (task.getProcess() != null) {
            upObj.setTaskId(task.getTaskId());
        }
        this.updateSnServer(upObj);
        final SnServerLog log = new SnServerLog(task);
        log.setOptName("安装服务器基础环境");
        log.setCommand("sudo curl " + sshUrl + " -o install.sh && chmod +x install.sh && sudo ./install.sh");
        this.snServerLogMapper.insertSnServerLog(log);
        final SysUser user = this.sysUserService.selectUserById(server.getCreateId());
        final String licenseSql = "INSERT INTO la_license (code, check_url, create_time) VALUES ('" + user.getLicense() + "', '" + this.checkLicenseUrl + "', sysdate());";
        log.setLogId((Long)null);
        log.setOptName("插入商户许可证");
        log.setCommand("sudo docker exec -it sync-mysql mysql -uroot -p" + dbPsd + " -D sync-client -e \"" + licenseSql + "\"");
        this.snServerLogMapper.insertSnServerLog(log);
        final String cloakSql = "update la_user_config set cloak_key = '" + user.getCloakKey() + "',cloak_id = '" + user.getCloakId() + "' where type = 0";
        log.setLogId((Long)null);
        log.setOptName("更新商户斗篷配置");
        log.setCommand("sudo docker exec -it sync-mysql mysql -uroot -p" + dbPsd + " -D sync-client -e \"" + cloakSql + "\"");
        this.snServerLogMapper.insertSnServerLog(log);
        if (this.dbSyncOpen) {
            log.setLogId((Long)null);
            log.setOptName("基础表结构同步");
            log.setCommand("");
            this.snServerLogMapper.insertSnServerLog(log);
            log.setLogId((Long)null);
            log.setOptName("更新爬虫UA库版本");
            log.setCommand("");
            this.snServerLogMapper.insertSnServerLog(log);
        }
        return Pair.of(task, (String)null);
    }
    
    public AjaxResult changeDomain(final SnServer snParam) {
        final SnServer server = this.selectSnServerByServerId(snParam.getServerId());
        if (server == null) {
            return AjaxResult.error("未查询到服务器信息");
        }
        if (!Objects.equals(server.getStatus(), 2)) {
            return AjaxResult.error("此服务器处于【" + this.dictDataService.selectDictLabel("server_status", server.getStatus().toString()) + "】状态，不能更换域名");
        }
        final String oldDomain = server.getDomain();
        final String domain = snParam.getDomain();
        if (Objects.equals(server.getDomain(), domain)) {
            return AjaxResult.warn("域名已应用到当前服务器，请勿重复操作");
        }
        final AjaxResult result = this.domainValidKernel.checkDomain(domain, server.getIp(), (Long)null);
        if (!result.isSuccess()) {
            return result;
        }
        String nginxConfig = FileUtils.readFile(this.baseNginxUrl);
        nginxConfig = nginxConfig.replace("#{server_name}", domain);
        nginxConfig = nginxConfig.replace("#{base_root}", "");
        nginxConfig = nginxConfig.replace("#{open_80}", "");
        final StrFile strFile = new StrFile();
        strFile.setContent(nginxConfig);
        strFile.setFileName(domain + ".conf");
        strFile.setContentType("text/plain");
        final R<SysFile> r1 = (R<SysFile>)this.remoteFileService.uploadStrToMinio(strFile);
        if (r1.getCode() != Constants.SUCCESS) {
            return AjaxResult.error(r1.getMsg());
        }
        final String nginxUrl = ((SysFile)r1.getData()).getUrl();
        final SnServer upObj = new SnServer();
        upObj.setServerId(server.getServerId());
        upObj.setDomain(domain);
        upObj.setNginxCnf(nginxUrl);
        upObj.setSslType(Integer.valueOf(0));
        this.updateSnServer(upObj);
        final SnServerTask task = this.createShellTaskKernel.changeBaseDomain(this.snServerMapper.selectSnServerByServerId(server.getServerId()), oldDomain);
        if (task != null) {
            this.serverTaskSender.sendOrderMessage(task.getTaskId());
        }
        return AjaxResult.success("更新中，请稍后查看服务器任务状态");
    }
    
    public AjaxResult deployDomainSSL(DeploySSLParams params) {
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        final SnServer server = this.selectSnServerByServerId(params.getId());
        if (server == null) {
            return AjaxResult.error("未查询到服务器信息");
        }
        if (!Objects.equals(server.getStatus(), 2)) {
            return AjaxResult.error("此服务器处于【" + this.dictDataService.selectDictLabel("server_status", server.getStatus().toString()) + "】状态，不能部署ssl证书");
        }
        if (params.getType() == 1) {
            final AjaxResult res = this.sslCountLimit.validCount(server.getDomain(), server.getIp());
            if (!res.isSuccess()) {
                return res;
            }
        }
        if (this.snServerTaskMapper.haveDealTask(server.getServerId(), "ssl证书", "0,1") > 0) {
            return AjaxResult.error("当前服务还有未执行完的ssl证书任务，请稍后再试");
        }
        final String domain = server.getDomain();
        SnServer upObj = new SnServer();
        upObj.setServerId(server.getServerId());
        if (params.getType() == 2) {
            upObj.setSslTime(SSLExpirationChecker.getCertificateExDate(params.getSslPem()));
        }
        String nginxConfig = FileUtils.readFile(this.baseNginxUrl);
        final Triple<AjaxResult, DeploySSLParams, String> triple = this.sslNginxConfigDealKernel.deal(params, server, domain, nginxConfig);
        if (!triple.getLeft().isSuccess()) {
            return triple.getLeft();
        }
        nginxConfig = triple.getRight();
        params = triple.getMiddle();
        nginxConfig = nginxConfig.replace("#{base_root}", "");
        upObj.setSslPem(params.getSslPem());
        upObj.setSslKey(params.getSslKey());
        upObj.setSslType(params.getType());
        final StrFile strFile = new StrFile();
        strFile.setContent(nginxConfig);
        strFile.setFileName(domain + ".conf");
        strFile.setContentType("text/plain");
        final R<SysFile> r1 = (R<SysFile>)this.remoteFileService.uploadStrToMinio(strFile);
        if (r1.getCode() != Constants.SUCCESS) {
            return AjaxResult.error(r1.getMsg());
        }
        final String nginxUrl = ((SysFile)r1.getData()).getUrl();
        upObj.setNginxCnf(nginxUrl);
        this.updateSnServer(upObj);
        final SnServerTask task = this.createShellTaskKernel.deploySSL(this.snServerMapper.selectSnServerByServerId(server.getServerId()), params);
        if (task != null) {
            this.serverTaskSender.sendOrderMessage(task.getTaskId());
            if (task.getProcess() != null) {
                upObj = new SnServer();
                upObj.setServerId(server.getServerId());
                upObj.setTaskId(task.getTaskId());
                this.updateSnServer(upObj);
            }
        }
        return AjaxResult.success("部署中，请稍后查看服务器任务状态");
    }
    
    public AjaxResult updateUI(final UpdateBaseParams params) {
        final SnServer server = this.selectSnServerByServerId(params.getId());
        if (server == null) {
            return AjaxResult.error("未查询到服务器信息");
        }
        if (!Objects.equals(server.getStatus(), 2)) {
            return AjaxResult.error("此服务器处于【" + this.dictDataService.selectDictLabel("server_status", server.getStatus().toString()) + "】状态，不能更新UI");
        }
        final SnTemplate temp = new SnTemplate();
        temp.getParams().put("limitNum", 1);
        temp.setCategory(Integer.valueOf(1));
        final List<SnTemplate> fonts = this.snTemplateMapper.selectSnTemplateList(temp);
        if (fonts.isEmpty()) {
            return AjaxResult.error("未获取到基础环境UI包，请联系管理员");
        }
        final SnTemplate font = fonts.get(0);
        if (Objects.equals(server.getUiVersion(), font.getVersion())) {
            return AjaxResult.error("当前已是最新版本");
        }
        params.setVersion(font.getVersion());
        params.setPackUrl(font.getPackUrl());
        final SnServer upObj = new SnServer();
        upObj.setServerId(server.getServerId());
        upObj.setUiVersion(font.getVersion());
        this.updateSnServer(upObj);
        final SnServerTask task = this.createShellTaskKernel.updateUI(this.snServerMapper.selectSnServerByServerId(server.getServerId()), params);
        if (task != null) {
            this.serverTaskSender.sendOrderMessage(task.getTaskId());
        }
        return AjaxResult.success("部署中，请稍后查看服务器任务状态");
    }
    
    public AjaxResult updateBack(final UpdateBaseParams params) {
        final SnServer server = this.selectSnServerByServerId(params.getId());
        if (server == null) {
            return AjaxResult.error("未查询到服务器信息");
        }
        if (!Objects.equals(server.getStatus(), 2)) {
            return AjaxResult.error("此服务器处于【" + this.dictDataService.selectDictLabel("server_status", server.getStatus().toString()) + "】状态，不能更新后台版本");
        }
        final String version = this.configService.selectConfigByKey("sync-client-version");
        if (Objects.equals(server.getVersion(), version)) {
            return AjaxResult.error("当前已是最新版本");
        }
        if (!Syncer.checkConnect(server.getIp() + ":3306", server.getDeDbPsd())) {
            return AjaxResult.error("操作失败，连接客户端数据库异常，请检查是否开放3306端口");
        }
        final SnServer upObj = new SnServer();
        upObj.setServerId(server.getServerId());
        upObj.setVersion(version);
        if (StringUtils.isEmpty(server.getUaVersion())) {
            upObj.setUaVersion("0.0");
        }
        final SnTemplate tmp = new SnTemplate();
        tmp.getParams().put("limitNum", 1);
        tmp.setCategory(Integer.valueOf(0));
        final List<SnTemplate> backs = this.snTemplateMapper.selectSnTemplateList(tmp);
        if (backs.isEmpty()) {
            return AjaxResult.error("未获取到基础环境后台包，请联系管理员");
        }
        final SnTemplate back = backs.get(0);
        if (!Objects.equals(server.getBackVersion(), back.getVersion())) {
            final UpdateBaseParams msParam = new UpdateBaseParams();
            msParam.setId(server.getServerId());
            msParam.setServeName("sync-message");
            final SnServerTask task1 = this.createShellTaskKernel.updateBack(server, msParam);
            if (task1 != null) {
                this.serverTaskSender.sendOrderMessage(task1.getTaskId());
            }
            params.setVersion(back.getVersion());
            params.setPackUrl(back.getPackUrl());
            params.setServeName("sync-client");
            final SnServerTask task2 = this.createShellTaskKernel.updateBack(this.snServerMapper.selectSnServerByServerId(server.getServerId()), params);
            if (task2 != null) {
                this.serverTaskSender.sendOrderMessage(task2.getTaskId());
                upObj.setBackVersion(back.getVersion());
            }
        }
        final SnTemplate temp = new SnTemplate();
        temp.getParams().put("limitNum", 1);
        temp.setCategory(Integer.valueOf(1));
        final List<SnTemplate> fonts = this.snTemplateMapper.selectSnTemplateList(temp);
        if (fonts.isEmpty()) {
            return AjaxResult.error("未获取到基础环境UI包，请联系管理员");
        }
        final SnTemplate font = fonts.get(0);
        if (!Objects.equals(server.getUiVersion(), font.getVersion())) {
            params.setVersion(font.getVersion());
            params.setPackUrl(font.getPackUrl());
            final SnServerTask task3 = this.createShellTaskKernel.updateUI(this.snServerMapper.selectSnServerByServerId(server.getServerId()), params);
            if (task3 != null) {
                this.serverTaskSender.sendOrderMessage(task3.getTaskId());
                upObj.setUiVersion(font.getVersion());
            }
        }
        if (!server.getPassword().endsWith("@gn")) {
            upObj.setEnPassword(server.getPassword());
        }
        if (!server.getDbPsd().endsWith("@gn")) {
            upObj.setEnDbPsd(server.getDbPsd());
        }
        this.updateSnServer(upObj);
        return AjaxResult.success("部署中，请稍后查看服务器任务状态");
    }
    
    public AjaxResult updateMessage(final UpdateBaseParams params) {
        final SnServer server = this.selectSnServerByServerId(params.getId());
        if (server == null) {
            return AjaxResult.error("未查询到服务器信息");
        }
        if (!Objects.equals(server.getStatus(), 2)) {
            return AjaxResult.error("此服务器处于【" + this.dictDataService.selectDictLabel("server_status", server.getStatus().toString()) + "】状态，不能更新后台版本");
        }
        final SnServerTask task = this.createShellTaskKernel.updateBack(server, params);
        if (task != null) {
            this.serverTaskSender.sendOrderMessage(task.getTaskId());
        }
        return AjaxResult.success("部署中，请稍后查看服务器任务状态");
    }
    
    public AjaxResult DDLUpdate(final Long serverId) {
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        final SnServer server = this.selectSnServerByServerId(serverId);
        if (server == null) {
            return AjaxResult.error("未查询到服务器信息");
        }
        if (!Objects.equals(server.getStatus(), 2)) {
            return AjaxResult.error("此服务器处于【" + this.dictDataService.selectDictLabel("server_status", server.getStatus().toString()) + "】状态，不能更新后台版本");
        }
        final SnServerTask task = new SnServerTask(server, loginUser);
        task.setTaskName("更新数据库表结构");
        task.setType(TaskTypeEnum.ddlUpdate.getType());
        this.snServerTaskMapper.insertSnServerTask(task);
        final SnServerLog log = new SnServerLog(task);
        log.setOptName("基础表结构同步");
        log.setCommand("");
        this.snServerLogMapper.insertSnServerLog(log);
        this.serverTaskSender.sendOrderMessage(task.getTaskId());
        return AjaxResult.success();
    }
    
    public AjaxResult syncUa(final Long serverId) {
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        final SnServer server = this.selectSnServerByServerId(serverId);
        if (server == null) {
            return AjaxResult.error("未查询到服务器信息");
        }
        if (!Objects.equals(server.getStatus(), 2)) {
            return AjaxResult.error("此服务器处于【" + this.dictDataService.selectDictLabel("server_status", server.getStatus().toString()) + "】状态，不能同步UA库");
        }
        if (StringUtils.isEmpty(server.getUaVersion())) {
            return AjaxResult.error("当前版本不支持UA过滤，请先更新后台版本");
        }
        final String version = this.configService.selectConfigByKey("client-ua-version");
        if (Objects.equals(server.getUaVersion(), version)) {
            return AjaxResult.error("当前已是最新版本");
        }
        if (!Syncer.checkConnect(server.getIp() + ":3306", server.getDeDbPsd())) {
            return AjaxResult.error("操作失败，连接客户端数据库异常，请检查是否开放3306端口");
        }
        final String[] sqlArr = this.filterUaService.generateInsertUaSql();
        if (sqlArr.length == 0) {
            return AjaxResult.error("UA库暂无数据，请联系管理员");
        }
        final SnServerTask task = new SnServerTask(server, loginUser);
        task.setBizVersion(server.getUaVersion());
        task.setTaskName("更新爬虫UA库版本 " + version);
        task.setType(TaskTypeEnum.uaUpdate.getType());
        this.snServerTaskMapper.insertSnServerTask(task);
        final SnServerLog log = new SnServerLog(task);
        log.setOptName("更新爬虫UA库版本");
        log.setCommand(server.getIp() + ":3306");
        this.snServerLogMapper.insertSnServerLog(log);
        this.serverTaskSender.sendOrderMessage(task.getTaskId());
        return AjaxResult.success("任务已提交，请稍后查看服务器任务状态");
    }
    
    public String syncUa(final SnServer server, final Boolean delCache) {
        if (server == null) {
            return "未查询到服务器信息";
        }
        final String version = this.configService.selectConfigByKey("client-ua-version");
        if (Objects.equals(server.getUaVersion(), version)) {
            return "当前已是最新版本";
        }
        final String[] sqlArr = this.filterUaService.generateInsertUaSql();
        if (sqlArr.length == 0) {
            return "UA库暂无数据，请联系管理员";
        }
        final AjaxResult res = Syncer.executeSql(server.getIp() + ":3306", server.getDeDbPsd(), sqlArr);
        if (!res.isSuccess()) {
            return String.valueOf(res.get((Object)"msg"));
        }
        final SnServer upObj = new SnServer();
        upObj.setServerId(server.getServerId());
        upObj.setUaVersion(version);
        this.snServerMapper.updateSnServer(upObj);
        if (delCache) {
            final ServiceShell shell = new ServiceShell();
            try {
                shell.connect(server.getIp(), server.getUserName(), server.getDePassword(), (int)server.getPort());
                shell.sendCommand("sudo docker exec -it sync-redis redis-cli DEL ua_cache_list");
            }
            catch (final Exception e) {
                return "UA库同步完成，缓存刷新失败，" + e.getMessage();
            }
            finally {
                shell.disconnect();
            }
        }
        return null;
    }
    
    public AjaxResult syncUaAll(final List<Long> serverIds) {
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        final SnServer snServer = new SnServer();
        snServer.setStatus(Integer.valueOf(2));
        snServer.setIsDelete(Integer.valueOf(0));
        List<SnServer> servers = this.snServerMapper.selectSnServerList(snServer);
        final String version = this.configService.selectConfigByKey("client-ua-version");
        SnServer server = null;
        servers = servers.stream().filter(server -> {
            if (StringUtils.isEmpty(server.getUaVersion())) {
                return false;
            }
            else {
                return !Objects.equals(server.getUaVersion(), version);
            }
        }).collect((Collector<? super Object, ?, List<SnServer>>)Collectors.toList());
        if (servers.isEmpty()) {
            return AjaxResult.warn("没有需要更新的服务器");
        }
        final String[] sqlArr = this.filterUaService.generateInsertUaSql();
        if (sqlArr.length == 0) {
            return AjaxResult.error("UA库暂无数据");
        }
        final Iterator<SnServer> iterator = servers.iterator();
        while (iterator.hasNext()) {
            server = iterator.next();
            final SnServerTask task = new SnServerTask(server, loginUser);
            task.setBizVersion(server.getUaVersion());
            task.setTaskName("更新爬虫UA库版本 " + version);
            task.setType(TaskTypeEnum.uaUpdate.getType());
            this.snServerTaskMapper.insertSnServerTask(task);
            final SnServerLog log = new SnServerLog(task);
            log.setOptName("更新爬虫UA库版本");
            log.setCommand(server.getIp() + ":3306");
            this.snServerLogMapper.insertSnServerLog(log);
            this.serverTaskSender.sendOrderMessage(task.getTaskId());
        }
        return AjaxResult.success("任务已提交，请稍后查看服务器任务状态");
    }
    
    @Async
    public void updateClientCloak(final Long userId) {
        try {
            final SysUser common = this.sysUserService.selectUserById(userId);
            final SnServer snServer = new SnServer();
            snServer.setCreateId(common.getUserId());
            snServer.setStatus(Integer.valueOf(2));
            snServer.setIsDelete(Integer.valueOf(0));
            final List<SnServer> servers = this.snServerMapper.selectSnServerList(snServer);
            servers.forEach(server -> {
                final String url = ((server.getSslType() == null || server.getSslType() == 0) ? "http://" : "https://") + server.getDomain() + "/api/user/config/updateCloak";
                final JSONObject params = new JSONObject();
                params.put((Object)"license", (Object)common.getLicense());
                params.put((Object)"cloakKey", (Object)common.getCloakKey());
                params.put((Object)"cloakId", (Object)common.getCloakId());
                final String res = new RequestUtil().doPostJson(url, params.toJSONString(new JSONWriter.Feature[0]));
                System.out.println("更新斗篷配置结果：" + res);
            });
        }
        catch (final Exception ex) {}
    }
    
    @Async
    public void logoutMerchant(final Long userId) {
        try {
            final SysUser common = this.sysUserService.selectUserById(userId);
            final SnServer snServer = new SnServer();
            snServer.setCreateId(common.getUserId());
            snServer.setStatus(Integer.valueOf(2));
            snServer.setIsDelete(Integer.valueOf(0));
            final List<SnServer> servers = this.snServerMapper.selectSnServerList(snServer);
            servers.forEach(server -> {
                final String url = ((server.getSslType() == null || server.getSslType() == 0) ? "http://" : "https://") + server.getDomain() + "/api/system/admin/logout";
                final JSONObject params = new JSONObject();
                params.put((Object)"license", (Object)common.getLicense());
                params.put((Object)"userName", (Object)common.getUserName());
                final String res = new RequestUtil().doPostJson(url, params.toJSONString(new JSONWriter.Feature[0]));
                System.out.println("修改密码下线客户端结果：" + res);
            });
        }
        catch (final Exception ex) {}
    }
    
    public SnServer selectSnServerByServerId(final Long serverId) {
        final SnServer obj = this.snServerMapper.selectSnServerByServerId(serverId);
        if (obj == null || obj.getIsDelete().equals(1)) {
            return null;
        }
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        if (loginUser.getRoles().contains("admin") || loginUser.getRoles().contains("super")) {
            return obj;
        }
        if (loginUser.getRoles().contains("dealer")) {
            return obj.getDealerId().equals(loginUser.getUserid()) ? obj : null;
        }
        if (loginUser.getRoles().contains("agent")) {
            return obj.getAgentId().equals(loginUser.getUserid()) ? obj : null;
        }
        return obj.getCreateId().equals(loginUser.getUserid()) ? obj : null;
    }
    
    public List<SnServer> selectSnServerList(final SnServer obj) {
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        if (loginUser.getRoles().contains("dealer")) {
            obj.setDealerId(loginUser.getUserid());
        }
        if (loginUser.getRoles().contains("agent")) {
            obj.setAgentId(loginUser.getUserid());
        }
        if (loginUser.getRoles().contains("common")) {
            obj.setCreateId(loginUser.getUserid());
        }
        obj.setIsDelete(Integer.valueOf(0));
        return this.snServerMapper.selectSnServerList(obj);
    }
    
    public AjaxResult insertSnServer(final SnServer snServer) {
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        if (!loginUser.getRoles().contains("admin") && !loginUser.getRoles().contains("super") && !loginUser.getRoles().contains("common")) {
            return AjaxResult.error("暂无权限添加服务器");
        }
        final Integer num = this.snServerMapper.getServerNum(loginUser.getUserid());
        final SysUser user = this.sysUserService.selectUserById(loginUser.getUserid());
        if (user.getServerLimit() != null && user.getServerLimit() <= num) {
            return AjaxResult.error("您当前最多可部署" + user.getServerLimit() + "个服务器，如需增加请联系管理员");
        }
        final SnServer query = new SnServer();
        query.setIp(snServer.getIp());
        query.setIsDelete(Integer.valueOf(0));
        if (!this.snServerMapper.selectSnServerList(query).isEmpty()) {
            return AjaxResult.error("已有相同IP的服务器，请勿重复添加");
        }
        snServer.setCreateId(loginUser.getUserid());
        snServer.setCreateName(loginUser.getUsername());
        if (loginUser.getRoles().contains("common")) {
            snServer.setAgentId(user.getAgentId());
            snServer.setAgentName(user.getAgentName());
            snServer.setDealerId(user.getDealerId());
            snServer.setDealerName(user.getDealerName());
        }
        snServer.setEnPassword(snServer.getPassword());
        snServer.setIsDelete(Integer.valueOf(0));
        snServer.setCreateTime(DateUtils.getNowDate());
        snServer.setUpdateTime(DateUtils.getNowDate());
        snServer.setProcess(Integer.valueOf(1001));
        return AjaxResult.success((Object)this.snServerMapper.insertSnServer(snServer));
    }
    
    public int updateSnServer(final SnServer snServer) {
        if (this.selectSnServerByServerId(snServer.getServerId()) == null) {
            return 0;
        }
        try {
            final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
            snServer.setUpdateId(loginUser.getUserid());
            snServer.setUpdateName(loginUser.getUsername());
        }
        catch (final Exception ex) {}
        snServer.setUpdateTime(DateUtils.getNowDate());
        snServer.setEnPassword(snServer.getPassword());
        snServer.setIp((String)null);
        snServer.setUpdateTime(DateUtils.getNowDate());
        return this.snServerMapper.updateSnServer(snServer);
    }
    
    public int deleteSnServerByServerIds(final Long[] serverIds) {
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        if (!loginUser.getRoles().contains("admin") && !loginUser.getRoles().contains("super")) {
            final Long createId = loginUser.getUserid();
            this.snServerTaskMapper.deleteSnServerTaskByServerIdV2(serverIds[0]);
            this.snServerTemplateMapper.deleteSnServerTemplateByServerIdsV2(serverIds);
            return this.snServerMapper.deleteSnServerByServerIdsV2(serverIds, createId);
        }
        this.snServerTaskMapper.deleteSnServerTaskByServerId(serverIds[0]);
        this.snServerTemplateMapper.deleteSnServerTemplateByServerIds(serverIds);
        return this.snServerMapper.deleteSnServerByServerIds(serverIds);
    }
    
    public AjaxResult getLogs(final DockerLogsParams params) {
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        if (!loginUser.getRoles().contains("admin") && !loginUser.getRoles().contains("super")) {
            return AjaxResult.error("暂无权限");
        }
        final SnServer server = this.snServerMapper.selectSnServerByServerId(params.getId());
        final ServiceShell shell = new ServiceShell(server.getServerId(), (ServerLogKernel)null);
        String logs = "";
        try {
            shell.connect(server.getIp(), server.getUserName(), server.getDePassword(), (int)server.getPort());
            if (params.getLine() == null) {
                params.setLine(Integer.valueOf(100));
            }
            logs = shell.executeCommand("sudo docker logs --tail " + params.getLine() + " " + params.getServeName());
        }
        catch (final Exception ex) {}
        finally {
            shell.disconnect();
        }
        return AjaxResult.success("获取成功", (Object)logs);
    }
}
